unit game_unit;

{
 sudoku analyser
}

interface

uses sysutils;

type TMove  = (mtFree,mtOrg,mtPlay);
     TSolveResult = (srNone,srNoSolution,srSolution);
     TField =  record
                mt      : TMove;
                move    : byte;
                options : word;
               end;

procedure Field2CR(var cc,rr : byte; fld :byte);
function CR2Field(cc,rr : byte) : byte;
function popcount(w : word) : byte;
procedure dropCRGOption(fld : byte; opt : word);
function SolveBruteForce(sres : TsolveResult) : TSolveResult;
procedure recalcOptions;
procedure analyseGame;
function searchZField : boolean;  //temp to test

var game : array[1..9,1..9] of TField;

implementation

uses unit1;

const maxtree = 200;
      continueMessage = 'druk GO of <spatiebalk> voor volgende';

var SH : word;                         //combinations shifter
    testX : array[1..9] of word;       //options test
    corrX : array[1..9] of word;       //corrections to testX
    selX  : array[1..9] of word;       //for green rectangles
//
    ZGame : array[1..9,1..9] of word;

//--  game position conversions --

function CR2Field(cc,rr : byte) : byte;
//convert column,row[1..9] to field [1..81]
begin
 if (cc=0) or (rr=0) then result := 0
  else result := cc + (rr-1)*9;
end;

procedure Field2CR(var cc,rr : byte; fld :byte);
//convert field[1..81] to colomn,row[1..9]
begin
 dec(fld);
 cc := (fld mod 9) + 1;
 rr := (fld div 9) + 1;
end;

function GroupCol(grp,ii:byte) : byte;
//supply column of field 1 group grp
begin
 result := ((grp-1) mod 3)*3 + ((ii-1) mod 3)+1;
end;

function GroupRow(grp,ii:byte) : byte;
//supply row of field 1 in  group grp
begin
 result := ((grp-1) div 3)*3 + ((ii-1) div 3)+1;
end;

function field2Group(fld : byte) : byte;
//return group of field fld
begin
 dec(fld);
 fld := fld div 3;
 result := (fld mod 3) + 3*(fld div 9) + 1;
end;

function GetRowField(rr,nr : byte) : byte;
//rr,nr : 1..9
begin
 result := 9*(rr - 1) + nr;
end;

function GetColumnField(cc,nr : byte) : byte;
//cc,rr : 1..9
begin
 result := 9*(nr-1) + cc;
end;

function GetGroupField(grp,nr: byte) : byte;
type TFld = record
             ff : byte;
             dd : byte;
            end;
const FLD : array[1..9] of TFld =
                           ((ff : 1; dd : 0),
                            (ff : 4; dd : 1),
                            (ff : 7; dd : 2),
                            (ff :28; dd : 9),
                            (ff :31; dd :10),
                            (ff :34; dd :11),
                            (ff :55; dd :18),
                            (ff :58; dd :19),
                            (ff :61; dd :20));
begin
 result := FLD[grp].ff + FLD[nr].dd;
end;

//-- game modifications --

procedure dropCRGOption(fld : byte; opt : word);
//drop option in column,row,group
var col,row,i,grp : byte;
begin
 opt := opt xor $3fe;  //complement
 field2CR(col,row,fld);
 for i := 1 to 9 do
  begin
   with game[i,row] do options := options and opt;
   with game[col,i] do options := options and opt;
  end;
 grp := field2Group(fld);
 for i := 1 to 9 do
  begin
   col := groupCol(grp,i);
   row := groupRow(grp,i);
   with game [col,row] do options := options and opt;
  end;
end;

function popcount(w : word) : byte;
//count bits set in w
var mask : word;
begin
 mask := $2;
 result := 0;
 while mask < $400 do
  begin
   if mask and w > 0 then inc(result);
   mask := mask shl 1;
  end;
end;

procedure resetSH(k : byte);
//set SH to first combination
var i : byte;
begin
 SH := 0;
 for i := 1 to k do SH := SH or (1 shl i);
end;

function advanceSH(k : byte) : boolean;
//SH shift counter increment
//k:choices  (1 bits set)
//return true if advanced
const N = 9;
var count,pos,i : byte;
    mask : word;
    Done : boolean;
begin
 result := false;
 count := 0;
 pos := N;
 Done := false;
 repeat
  mask := (1 shl pos);
  if SH and mask > 0 then                   //test SH bit
   begin
    SH := SH xor mask;                      //remove 1
    inc(count);                             //count removed 1
    if pos + count <= N then                //test space to shift
     begin
      mask := mask shl 1;                   //next bit
      Done := true;                         
      result := true;
     end
     else
      if count = K then Done := true
       else dec(pos);
   end
   else dec(pos);
 until Done;
 for i := count downto 1 do                 //add removed bits
  begin
   SH := SH xor mask;
   mask := mask shl 1;
  end;
end;

procedure recalcOptions;
//recalculate all option fields
var rowopts : array[1..9] of word;
    colopts : array[1..9] of word;
    grpopts : array[1..9] of word;
    cc,rr,i,grp : byte;
    mask : word;
begin
 for i := 1 to 9 do
  begin
   colopts[i] := $3fe;
   rowopts[i] := $3fe;
   grpopts[i] := $3fe;
  end;
 for i := 1 to 81 do
  begin
   grp := field2Group(i);
   field2CR(cc,rr,i);
   with game[cc,rr] do
    if move <> 0 then begin
                       mask := (1 shl move) xor $3fe;
                       colopts[cc] := colopts[cc] and mask;
                       rowopts[rr] := rowopts[rr] and mask;
                       grpopts[grp] := grpopts[grp] and mask;
                      end;
  end;
 for i := 1 to 81 do
  begin
   grp := field2Group(i);
   field2CR(cc,rr,i);
   with game[cc,rr] do
    if move = 0 then options := colopts[cc] and rowopts[rr] and grpopts[grp]
     else options := 1 shl move;
  end;
end;

//-- brute force solutions --

function MoveOK(fld, m:byte) : boolean;
//return true if move m OK in field m
//search column,row,group for same nr
//brute force search does not use option fields
var cc,rr,gp,i : byte;
    mask,sum,b : word;
begin
 result := true;
 field2CR(cc,rr,fld);
 gp := field2Group(fld);
 mask := 1 shl m;
 sum := 0;
 for i := 1 to 9 do
  begin
   b := 1 shl game[cc,i].move;
   sum := sum or b;
  end;
 if (sum and mask) > 0 then
  begin
   result := false;
   exit;
  end;

 sum := 0;
 for i := 1 to 9 do
  begin
   b := 1 shl game[i,rr].move;
   sum := sum or b;
  end;
 if (sum and mask) > 0 then
  begin
   result := false;
   exit;
  end;

 sum := 0;
 for i := 1 to 9 do
  begin
   cc := groupCol(gp,i);
   rr := groupRow(gp,i);
   b := 1 shl game[cc,rr].move;
   sum := sum or b;
  end;
 if (sum and mask) > 0 then result := false;
end;

function SolveBruteForce(sres : TsolveResult) : TSolveResult;
//XF is trial field
//XM is trial move
//only affect move in fields with movetype = mtSolve
var XF,XM,cc,rr : byte;
label loop1,loop2,nextXF,nextXM,prevXF;
begin
 case sres of                       //define start position
  srNosolution : begin
                  result := srNoSolution;
                  exit;
                 end;
  srSolution   : begin
                  XF := 82;
                  XM := 9;
                  field2CR(cc,rr,XF);
                  goto prevXF;
                 end;
  end;//case

 XF := 1; XM := 1;
 field2CR(cc,rr,XF);

//  test for free field

loop1:

 if game[cc,rr].mt <> mtFree then goto nextXF;

loop2:

 if moveOK(XF,XM) = false then goto nextXM;
 game[cc,rr].move := XM;

nextXF:

 if XF = 81 then
  begin
   result := srSolution;
   exit;
  end;
 inc(XF);
 field2CR(cc,rr,XF);
 XM := 1;
 goto loop1;

nextXM:

 if XM < 9 then
  begin
   inc(XM);
   goto loop2;
  end;

prevXF:

 if (XF=1) then
  begin
   result := srNoSolution;
   exit;
  end;
//
 dec(XF);
 field2CR(cc,rr,XF);
 if game[cc,rr].mt <> mtFree then goto prevXF;
 XM := game[cc,rr].move;
 game[cc,rr].move := 0;
 goto nextXM;

end;

//-- analyse  helpers

procedure loadXRow(row : byte);
var i : byte;
begin
 for i := 1 to 9 do testX[i] := game[i,row].options;
end;

procedure storeXRow(row : byte);
var i : byte;
begin
 for i := 1 to 9 do game[i,row].options := testX[i];
end;

procedure loadXColumn(col : byte);
var i : byte;
begin
 for i := 1 to 9 do testX[i] := game[col,i].options;
end;

procedure storeXColumn(col : byte);
var i : byte;
begin
 for i := 1 to 9 do game[col,i].options := testX[i];
end;

procedure loadXGroup(grp : byte);
var col,row,i : byte;
begin
 for i := 1 to 9 do
  begin
   col := GroupCol(grp,i);
   row := GroupRow(grp,i);
   testX[i] := game[col,row].options;
  end;
end;

procedure storeXGroup(grp : byte);
var col,row,i : byte;
begin
 for i := 1 to 9 do
  begin
   col := GroupCol(grp,i);
   row := GroupRow(grp,i);
   game[col,row].options :=  testX[i];
  end;
end;

procedure loadX(nr : byte);
//load Xtest word; nr: 1..9:row;  10..18:column;  19..27:group
begin
 if nr < 10 then loadXrow(nr)
  else if nr < 19 then loadXcolumn(nr-9)
        else loadXgroup(nr-18);
end;

procedure storeX(nr : byte);
begin
 if nr < 10 then storeXrow(nr)
  else if nr < 19 then storeXcolumn(nr-9)
        else storeXgroup(nr-18);
end;

function Solution : boolean;
//check for solution
//exit true if solution
var i,j : byte;
    sum,w : word;
begin
 result := true;
 j := 1;
 while result and (j <= 9) do
  begin
   sum := 0;
   i := 1;
   while result and (i <= 9) do
    begin
     w := game[i,j].options;
     result := (sum and w) = 0;
     sum := sum or w;
     inc(i);
    end;
   inc(j);
  end;
end;

procedure repaintX(nr : byte);
//repaint the fields of group nr (1..27)
var i,cc,rr,gp : byte;
begin
 if nr < 10 then
  begin
   rr := nr;
   for i := 1 to 9 do repaintfield(getRowField(rr,i));
  end
 else
  if nr < 19 then
   begin
    cc := nr - 9;
    for i := 1 to 9 do repaintfield(getColumnField(cc,i));
   end
  else
   begin
    gp := nr - 18;
    for i := 1 to 9 do repaintfield(getGroupField(gp,i));
   end;
end;

function X2text(nr : byte) : string;
//nr 1..27  row,column,group
//supply string 'column n: ' etc.
begin
 if nr < 10 then result := 'rij '
  else if nr < 19 then
   begin
    result := 'kolom ';
    nr := nr - 9;
   end
    else begin
          nr := nr - 18;
          result := 'groep ';
         end;
 result := result + inttostr(nr)+':';
end;

function SH2text(SH1 : word) : string;
var i : byte;
begin
  result := ' Velden[';
 for i := 1 to 9 do
  if ((1 shl i) and SH1) > 0 then result := result + inttostr(i) + ',';
 delete(result,length(result),1);
 result := result + ']';
 i := popcount(SH1);
 if i = 1 then result := result + ' (veld met 1 optie)';
 if i = 8 then result := result + ' (enig veld voor optie)';
end;

procedure showcorrections(nr : byte);
//paint red rectangle on options to be removed
//nr is 9 field group 1..9  10..18  19..27
var i,fld,m : byte;
    mask : word;
begin
 fld := 0;
 for i := 1 to 9 do
  begin
   case nr of
     1..9  : begin
              fld := GetRowField(nr,i);
             end;
    10..18 : begin
              fld := GetColumnField(nr-9,i);
             end;
    19..27 : begin
              fld := GetGroupField(nr-18,i);
             end;
   end;//case
   for m := 1 to 9 do
    begin
     mask := 1 shl m;
     if (mask and corrX[i]) > 0 then crossOption(fld,m);
     if (mask and selX[i] ) > 0 then paintOptionMark(fld,m,$00d000); //green
    end;
  end;//for
end;

procedure Xreport(code,X : byte);
//code 1:insufficient options  2:B reduce  3:A reduce
//report option reduction by analyse process
//pause
var HH : word;
begin
 if code = 1 then form1.msglabel.Caption := 'ontbrekende opties  '+ X2Text(X)
  else
   begin
    if code = 2 then HH := SH else HH := SH xor $3fe;
    showCorrections(X);
    form1.msglabel.Caption := X2Text(X)+' '+SH2Text(HH);
   end;
 pause;  //to show reductions
end;

procedure Xreduce(nr : byte);
//reduce Xtest[..] and restore
//pause
var i : byte;
begin
 for i := 1 to 9 do testX[i] := testX[i] xor corrX[i];
 storeX(nr);
 repaintX(nr);
 form1.msglabel.Caption := continueMessage;
 pause;  //to show result
end;

//-- analyse  (field interactions within columns, rows, groups)

function analyseX(K,X : byte) : boolean;
//exit true if modification
//analyse 9 bit group X for K bit combinations
var sumA,sumB,mask,XChange : word;
    i,A,B : byte;
begin
 result := false;
 loadX(X);
 resetSH(K);
 repeat
  sumA := 0;
  sumB := 0;
  for i := 1 to 9 do
   begin
    mask := 1 shl i;
    if (mask and SH) > 0 then sumA := sumA or testX[i]
     else sumB := sumB or testX[i];
   end;
  A := popcount(sumA);
  B := popcount(sumB);
//
  if (A=0) or (B=0) or ((A+B)<9) then
   begin
    Xreport(1,X);                       //insufficient options
    result := true;
    exit;
   end;
//
  for i := 1 to 9 do
   begin
    corrX[i] := 0;
    selX[i] := 0;
   end;
  Xchange := 0;
  if A = K then
   begin
    for i := 1 to 9 do
     begin
      mask := 1 shl i;
      if (mask and SH) = 0 then
       begin
        corrX[i] := testX[i] and sumA;
        Xchange := Xchange or corrX[i];
       end
       else
        begin
         selX[i] := testX[i] and sumA;
        end;
     end;
    if XChange > 0 then
     begin
      Xreport(2,X);   //A bits correction
      XReduce(X);
      result := true; //exit restart
     end;
   end else if B = 9-K then
             begin
              for i := 1 to 9 do
               begin
                mask := 1 shl i;
                if (mask and SH) > 0 then
                 begin
                  corrX[i] := testX[i] and sumB;
                  XChange := Xchange or corrX[i];
                  selX[i] := testX[i] and (sumB xor $3fe);
                 end;
               end;
              if XChange > 0 then
               begin
                Xreport(3,X);   //B bits correction
                Xreduce(X);
                result := true; //exit restart
               end; 
             end;
 until (advanceSH(K) = false) or ExitFlag or (result = true);
end;

//-- triples processing (group interactions)

procedure paintgreenOptions(cc,rr : byte; w : word);
//paint green rectangles around 1 bits in w for field fld
var i,fld : byte;
begin
 fld := CR2Field(cc,rr);
 for i := 1 to 9 do
  if (1 shl i) and w > 0 then paintOptionMark(fld,i,$00d000);
end;

procedure paintCrossOptions(cc,rr : byte; w : word);
//paint red rect and X cross thyru option fields
var i,fld : byte;
begin
 fld := CR2Field(cc,rr);
 for i := 1 to 9 do
  if (1 shl i) and w > 0 then crossOption(fld,i);
end;

function CheckHTriple(tt,rr : byte) : boolean;
//check horizontal triple
//report modifications
//return true if modified
var col,row,i,j,H,x,y : byte;
    SumA,SumB,C,NC,w : word;
    s : string;
begin
 SumA := 0;
 SumB := 0;
 y := ((rr-1) div 3)*3;  //triple top
 x := (tt-1)*3;          //triple left
 for i := 1 to 9 do
  begin
   H := (i-1) div 3 + 1; //hor triple
   if H <> tt then sumA := sumA or game[i,rr].options;
  end;
 for j := 1 to 3 do
  begin
   row := y + j;
   if row <> rr then
    for i := 1 to 3 do
     begin
      col := x + i;
      sumB := sumB or game[col,row].options;
     end;
  end;
 C := sumA and sumB;
 if (sumA xor sumB) = 0 then
  begin
   result := false;
   exit;             //exit if no reduction
  end
  else result := true;
//
 NC := C xor $3fe;   //not C
 for i := 1 to 9 do                  //horizontal markers green/red
  begin
   H := (i-1) div 3 + 1;
   w := game[i,rr].options and NC;
   if w > 0 then
    if H = tt then begin
                    if game[i,rr].mt = mtfree then
                     paintGreenOptions(i,rr,w);
                   end
     else paintCrossOptions(i,rr,w);
  end;
//
 for j := 1 to 3 do
  begin
   row := y+j;
   if row <> rr then
    for i := 1 to 3 do
     begin
      col := x+i;
      w := game[col,row].options and NC;
      if w > 0 then paintcrossOptions(col,row,w);
     end;//for i
  end;//for j
//
 s := 'rij['+inttostr(rr)+ '] ';
 s := s + 'drietal['+inttostr(tt)+']';
 form1.msglabel.Caption := s;
 pause;                         //show changes
// implement changes
 for i := 1 to 9 do                  //horizontal reductions
  begin
   H := (i-1) div 3 + 1;
   if H <> tt then game[i,rr].options := game[i,rr].options and C;
  end;
 for j := 1 to 3 do
  begin
   row := y+j;
   if row <> rr then
    for i := 1 to 3 do
     begin
      col := x+i;
      game[col,row].options := game[col,row].options and C;
     end;//for i
  end;//for j
 repaintgame;
 form1.msglabel.Caption := continueMessage;
 pause;               //show reduction
end;

function checkVtriple(cc,tt : byte) : boolean;
//check, display,reduce vertical triple tt at column cc
var col,row,i,j,V,x,y : byte;
    SumA,SumB,C,NC,w : word;
    s : string;
begin
 SumA := 0;
 SumB := 0;
 y := (tt-1)*3;   //vertical bias
 x := ((cc-1) div 3)*3;
 for j := 1 to 9 do                    //make sumA
  begin
   V := ((j-1) div 3) + 1;
   if V <> tt then sumA := sumA or game[cc,j].options;
  end;
 for i := 1 to 3 do
  begin                                //make sumB
   col := x + i;
   for j := 1 to 3 do
    begin
     row := y + j;
     if col <> cc then sumB := sumB or game[col,row].options;
    end;//for j
  end;//for i
 C := sumA and sumB;
 if (sumA xor sumB) = 0 then
  begin
   result := false;
   exit;             //exit if no reduction
  end
   else result := true;
//
 NC := C xor $3fe;             //missing options "1"
 for j := 1 to 9 do            //show red & green markers
  begin
   V := ((j-1) div 3) + 1;
   w := game[cc,j].options and NC;
   if w > 0 then
    if V = tt then begin
                    if game[cc,j].mt = mtFree then 
                      paintGreenOptions(cc,j,w);
                   end
     else paintCrossOptions(cc,j,w);
  end;
 for i := 1 to 3 do
  begin
   col := x + i;
   if col <> cc then
    for j := 1 to 3 do
     begin
      row := y + j;
      w := game[col,row].options and NC;
      if w > 0 then paintCrossOptions(col,row,w);
     end;
  end;
//
 s := 'kolom['+inttostr(cc)+ '] ';
 s := s + 'drietal['+inttostr(tt)+']';
 form1.msglabel.Caption := s;
 pause;          //show selections
//
 for j := 1 to 9 do
  begin
   V := ((j-1) div 3) + 1;
   if V <> tt then game[cc,j].options := game[cc,j].options and C;
  end;
 for i := 1 to 3 do
  begin
   col := x + i;
   for j := 1 to 3 do
    begin
     row := y + j;
     if col <> cc then game[col,row].options := game[col,row].options and C;
    end;//for j
  end;//for i
 repaintgame;
 form1.msglabel.Caption := continueMessage;
 pause;
end;

//--- Z reduction:  enter number --> reduce --> check zero option fields
//                  true: report wrong  option, empty field --> remove option

procedure loadZoptions;
var i,j : byte;
begin
 for j := 1 to 9 do
  for i := 1 to 9 do ZGame[i,j] := Game[i,j].options;
end;

function reduceXZ(K : byte) : boolean;
//reduce testX for SH bits set
//exit true if change in testX
var A,B,i: byte;
    sumA,sumB,sumAC,sumBC,change : word;
begin
 resetSH(K);
 change := 0;
 repeat                  //advanceSH loop
  sumA := 0; sumB := 0;
  for i := 1 to 9 do
   if ((1 shl i) and SH) > 0 then sumA := sumA or testX[i]
    else sumB := sumB or testX[i];
  A := popcount(sumA);
  B := popcount(sumB);
  if A = K then
   begin
    sumAC := sumA xor $3fe;
    for i := 1 to 9 do
     if ((1 shl i) and SH) = 0 then
      begin
       change := change or (testX[i] and sumA);
       testX[i] := testX[i] and sumAC;
      end;
   end;
  if B = 9-K then
   begin
    sumBC := sumB xor $3fe;
    for i := 1 to 9 do
     if ((1 shl i) and SH) > 0  then
      begin
       change := change or (testX[i] and sumB);
       testX[i] := testX[i] and sumBC;
      end;
   end;
 until advanceSH(K) = false;
 result := change > 0;
end;

function checkZrows(var zf : byte; K : byte) : boolean;
//test all rows
//true exit:  change took place
var row,i : byte;
    change1,change2 : boolean;
begin
 row := 1;
 change2 := false;
 repeat        //rows 1..9
  for i := 1 to 9 do testX[i] := ZGame[i,row];
  change1 := reduceXZ(K);
  if change1 then
   for i := 1 to 9 do
    begin
     if testX[i] = 0 then zf := CR2Field(i,row);
     ZGame[i,row] := testX[i];                  //updata ZGame
    end;
  change2 := change2 or change1;
  inc(row);
 until (row > 9) or (zf > 0);
 result := (zf > 0) or change2;
end;

function checkZcolumns(var zf : byte; K : byte) : boolean;
//true exit:  change took place
var col,j : byte;
    change1,change2 : boolean;
begin
 col := 1;
 change2 := false;
 repeat
  for j := 1 to 9 do testX[j] := ZGame[col,j];
  change1 := reduceXZ(K);
  if change1 then
   for j := 1 to 9 do
    begin
     if testX[j] = 0 then zf := CR2Field(col,j);
     ZGame[col,j] := testX[j];                  //updata ZGame
    end;
  change2 := change2 or change1;
  inc(col);
 until (col > 9) or (zf > 0);
 result := (zf > 0) or change2;
end;

function checkZgroups(var zf : byte; K : byte) : boolean;
//true exit:  change took place
var grp,cc,rr,i: byte;
    change1,change2 : boolean;
begin
 grp := 1;
 change2 := false;
 repeat                 //for all groups
  for i := 1 to 9 do    //load group into testX
   begin
    cc := GroupCol(grp,i);
    rr := GroupRow(grp,i);
    testX[i] := ZGame[cc,rr];
   end;
  change1 := reduceXZ(K);
  if change1 then
   for i := 1 to 9 do    //restore testX in ZGame
    begin
     cc := GroupCol(grp,i);
     rr := Grouprow(grp,i);
     if testX[i] = 0 then zf := CR2Field(cc,rr);
     ZGame[cc,rr] := testX[i];                 //update ZGame
    end;
  change2 := change2 or change1;
  inc(grp);
 until (zf > 0) or (grp > 9);
 result := (zf > 0) or change2;
end;

procedure dropZoption(cc,rr : byte; mopt : word);
//drop bits of mopt in column, row and group
var i,fld,col,row,grp : byte;
begin
 mopt := mopt xor $3fe;
 fld := CR2Field(cc,rr);
 grp := Field2Group(fld);
 for i := 1 to 9 do
  begin
   ZGame[cc,i] := ZGame[cc,i] and mopt;
   ZGame[i,rr] := ZGame[i,rr] and mopt;
   fld := getGroupField(grp,i);
   Field2CR(col,row,fld);
   ZGame[col,row]:= Zgame[col,row] and mopt;
  end;
end;   

function searchZField : boolean;
//search for Zero field
//all fields, all options
//exit true if found
//report fields/options, make game[] corrections
var cc,rr,opt,i,j,n,f,fld,ZField,K : byte;
    mopt,wopt : word;
    restart,ZF : boolean;
begin
 Zfield := 0; opt := 0; mopt := 0;
 ZF := false;
 K := 1;
 repeat
  fld := 1;
  repeat                              //fields 1..81 loop
   Field2CR(cc,rr,fld);
   wopt := game[cc,rr].options;
   if popcount(wopt) > 1 then
    begin
     opt := 1;
     repeat                          //options 1..9
      mopt := 1 shl opt;             //option bit mask
      if (mopt and wopt) > 0 then
       begin
        loadZOptions;                //game[] --> ZGame[]
        dropZoption(cc,rr,mopt);
        ZGame[cc,rr] := mopt;
        repeat                      //row,col,group
         i := 1;
         restart := false;
         repeat
          case i of
           1 : restart := restart or checkZrows(Zfield,K);
           2 : restart := restart or checkZcolumns(Zfield,K);
           3 : restart := restart or checkZgroups(Zfield,K);
          end;//case
          inc(i);
          ZF := Zfield > 0;
         until (i > 3) or ZF;
        until (restart = false) or ZF;
       end;//if wopt &mopt
      if ZF = false then inc(opt);
     until (opt > 9) or ZF;
    end;//if popcount
   if ZF = false then inc(fld);
  until ZF or (fld>81);
  inc(K);
 until (K > 4) or ZF;
//
 if ZF then
  begin
   for j := 1 to 9 do
    for i := 1 to 9 do
     begin
      wopt := ZGame[i,j] xor Game[i,j].options;
      f := CR2Field(i,j);
      if f <> fld then
       for n := 1 to 9 do
        if ((1 shl n) and wopt) > 0 then crossOption(f,n);
     end;
   paintredNr(fld,opt);
   paintCursor(ZField,$ff);
   form1.msglabel.Caption := 'kolom '+inttostr(cc) +
                             ' rij '+inttostr(rr) +
                             ' optie '+inttostr(opt) +
                             ' --> nul optie veld';
   pause;
   form1.msglabel.Caption := continuemessage;
   wopt := mopt xor $3fe;
   game[cc,rr].options := game[cc,rr].options and wopt;
   repaintgame;
   pause;
   result := true;
  end
  else result := false;
end;

//--- main call & control ---

procedure analyseGame;
var K : byte;    //bit count in combination
    X : byte;    //9 digit group 1..27
    Restart : boolean;
    triple : byte;
    col,row : byte;
label Lrestart,Lsolved,Lexit;
begin
 exitflag := false;
 stopflag := false;

Lrestart :

  restart := false;
  K := 1;
  repeat                       //K (combination count) loop
   X := 1;
   repeat                      //block loop 1..27
    if analyseX(K,X) then goto Lrestart      //analyseX : true if modification
     else inc(X);
   until (X = 28) or ExitFlag;
   inc(K);
  until (K = 5) or ExitFlag;
//
  if solution then  goto Lsolved;
  if exitflag then goto Lexit;
//
  row := 1;              //horizontal triples check
  repeat
   triple := 1;
   repeat
    restart := restart or CheckHTriple(triple,row);
    inc(triple);
   until (triple > 3) or exitflag;
   inc(row);
  until (row=10) or restart or exitflag;
//
  if exitflag then goto Lexit;
  if solution then goto Lsolved;
  if restart then goto Lrestart;
//
  col := 1;              //vertical triples check
  repeat
   triple := 1;
   repeat
    restart := restart or CheckVtriple(col,triple);
    inc(triple);
   until (triple > 3) or exitflag;
   inc(col);
  until (col = 10) or exitflag;
//
  if exitflag then goto Lexit;
  if solution then goto Lsolved;
  if restart then goto Lrestart;
//
  restart := searchZField;
  if exitflag then goto Lexit;
  if restart then goto Lrestart;
  if solution then goto Lsolved;
  exit;
//

Lsolved:

 form1.msglabel.Caption := 'spel opgelost';
 pause;
 exit;

Lexit:

  form1.msglabel.caption := ' einde analyse';
  pause;
end;

end.
